
init()
{
	if(!level.ex_kc) return;

	level.kc_array = [];

	// lock slot 0 because kcAllocate should issue slot numbers above 0 (self.ex_confirmkill 0 means no KC)
	level.kc_array[0] = spawnstruct();
	level.kc_array[0].inuse = 1;

	// set an entity max to prevent overload
	level.kc_maxent = 64;
}

kcCheck(attacker, sMeansOfDeath, sWeapon)
{
	// check if this kill has to be confirmed
	if(level.ex_kc && isPlayer(attacker) && attacker != self && (!level.ex_teamplay || attacker.pers["team"] != self.pers["team"]))
	{
		switch(level.ex_kc_weapon)
		{
			// all weapons
			case 0:
				return(kcAllocate());
			// all weapons, except gunship
			case 1:
				if((level.ex_gunship && isPlayer(level.gunship.owner) && level.gunship.owner == attacker) ||
				   (level.ex_gunship_special && isPlayer(level.gunship_special.owner) && level.gunship_special.owner == attacker)) return(0);
				return(kcAllocate());
			// all weapons, except gunship and snipers
			case 2:
				if((extreme\_ex_weapons::isWeaponType(sWeapon, "sniper") && sMeansOfDeath != "MOD_MELEE") ||
				  (level.ex_gunship && isPlayer(level.gunship.owner) && level.gunship.owner == attacker) ||
				  (level.ex_gunship_special && isPlayer(level.gunship_special.owner) && level.gunship_special.owner == attacker)) return(0);
				return(kcAllocate());
			// all main weapons (from weapon menus)
			case 3:
				if(maps\mp\gametypes\_weapons::isMainWeapon(sWeapon)) return(kcAllocate());
				return(0);
			// all main weapons, except snipers
			case 4:
				if(!maps\mp\gametypes\_weapons::isMainWeapon(sWeapon)) return(0);
				if(extreme\_ex_weapons::isWeaponType(sWeapon, "sniper") && sMeansOfDeath != "MOD_MELEE") return(0);
				return(kcAllocate());
		}
	}

	// no KC
	return(false);
}

kcAllocate()
{
	if(level.kc_array.size == level.kc_maxent) return(0);

	for(i = 0; i < level.kc_array.size; i++)
	{
		if(level.kc_array[i].inuse == 0)
		{
			level.kc_array[i].inuse = 1;
			kcStatsInit(i);
			return(i);
		}
	}

	level.kc_array[i] = spawnstruct();
	level.kc_array[i].inuse = 1;
	kcStatsInit(i);
	return(i);
}

kcStatsInit(index)
{
	level.kc_array[index].attacker_stats = [];
	level.kc_array[index].victim_stats = [];
}

kcStatsAttacker(index, stat)
{
	kc_record = level.kc_array[index];
	kc_record.attacker_stats[kc_record.attacker_stats.size] = stat;
}

kcStatsVictim(index, stat)
{
	kc_record = level.kc_array[index];
	kc_record.victim_stats[kc_record.victim_stats.size] = stat;
}

kcMain(regular_points, reward_points, isteamscore, attacker)
{
	index = self.ex_confirmkill;

	if(!regular_points && !reward_points)
	{
		kcFree(index);
		return;
	}

	level.kc_array[index].points = regular_points;
	level.kc_array[index].reward = reward_points;
	level.kc_array[index].isteamscore = isteamscore;
	level.kc_array[index].victim = self;
	level.kc_array[index].victim_team = self.pers["team"];
	level.kc_array[index].attacker = attacker;

	if(self.pers["team"] == "axis") model = game["dogtag_axis"];
		else model = game["dogtag_allies"];

	level.kc_array[index].dogtags = spawn("script_model", self.origin + (0,0,20));
	level.kc_array[index].dogtags setmodel(model);
	level.kc_array[index].dogtags.angles = (0, self.angles[1], 0);
	level.kc_array[index].trigger = spawn("trigger_radius", level.kc_array[index].dogtags.origin, 0, 20, 50);

	// play hint to get tags once
	if(isPlayer(attacker) && !isDefined(attacker.kc_gettags))
	{
		attacker.kc_gettags = true;
		attacker playLocalSound("kc_gettags");
	}

	// Start bounce and rotation
	level.kc_array[index].dogtags thread dogtagRotate();

	// Remove the trigger and dogtags if the dog tag expire
	level.kc_array[index].dogtags thread dogtagTimeoutMonitor(index, level.ex_kc_timeout);

	// Wait for another player to pickup the dogtags
	level.kc_array[index].dogtags thread dogtagPickupMonitor(index);
}

kcFree(index)
{
	if(!isDefined(level.kc_array[index])) return;
	if(isDefined(level.kc_array[index].trigger)) level.kc_array[index].trigger delete();
	if(isDefined(level.kc_array[index].dogtags)) level.kc_array[index].dogtags delete();
	level.kc_array[index].inuse = 0;
}

dogtagRotate()
{
	self endon("kc_endtimeout");
	self endon("kc_endpickup");

	while(true)
	{
		self movez(20, 1.5, 0.3, 0.3);
		self rotateyaw(360, 1.5, 0, 0);
		wait( 1.5 );
		self movez(-20, 1.5, 0.3, 0.3);
		self rotateyaw(360 ,1.5, 0, 0);
		wait( 1.5 );
	}
}

dogtagTimeoutMonitor(index, timeout)
{
	self endon("kc_endtimeout");

	// Wait for this dog tag to timeout
	wait( [[level.ex_fpstime]](timeout) );

	// Notify trigger monitor to end
	self notify("kc_endpickup");

	// Delete trigger and model
	kcFree(index);
}

dogtagPickupMonitor(index)
{
	self endon("kc_endpickup");

	kc_record = level.kc_array[index];

	while(true)
	{
		kc_record.trigger waittill("trigger", player);

		// make sure that victim and attacker variables are valid entities
		if(!isPlayer(kc_record.victim)) kc_record.victim = self;
		if(!isPlayer(kc_record.attacker)) kc_record.attacker = self;

		// pickup by friendlies (the one killed or his teammates)
		if( (!level.ex_teamplay && player == kc_record.victim) || (level.ex_teamplay && player.pers["team"] == kc_record.victim_team) )
		{
			// if pickup by friendlies is disabled, deny pickup
			if(!level.ex_kc_denied) continue;

			// if only victim can collect, and you're not, deny pickup
			if((level.ex_kc_denied == 1 || level.ex_kc_denied == 2) && player != kc_record.victim) continue;

			// handle bonus points
			points_denied = kc_record.points + kc_record.reward;
			if(level.ex_kc_denied == 5 || (level.ex_kc_denied == 4 && player == kc_record.victim))
			{
				points = level.ex_kc_denied_bonus;
				player.pers["score"] += points;
				player.score += points;
				player.pers["bonus"] += points;
				player notify("update_playerscore_hud");
				if(!level.ex_teamplay) player [[level.checkscorelimit]]();

				if(kc_record.isteamscore)
				{
					teamscore = getTeamScore(player.pers["team"]);
					teamscore += points;
					setTeamScore(player.pers["team"], teamscore);
					[[level.checkscorelimit]]();
				}
			}

			// sounds and messages
			if(level.ex_kc_denied_msg)
			{
				// for collector
				if((level.ex_kc_denied_msg & 1) == 1) player playLocalSound("kc_denied");
				if(player != kc_record.victim)
				{
					if((level.ex_kc_denied_msg & 2) == 2) player thread dogtagConfirmHUD(&"MISC_KDC_HUD");

					if(isPlayer(kc_record.victim))
					{
						if((level.ex_kc_denied_msg & 4) == 4) player iprintln(&"MISC_KDC_FOR", [[level.ex_pname]](kc_record.victim));
						if((level.ex_kc_denied_msg & 1) == 1) kc_record.victim playLocalSound("kc_denied");
						if((level.ex_kc_denied_msg & 2) == 2) kc_record.victim thread dogtagConfirmHUD(&"MISC_KDV_PROXY_HUD");
						if((level.ex_kc_denied_msg & 4) == 4) kc_record.victim iprintln(&"MISC_KDV_BY", [[level.ex_pname]](player));
						//if((level.ex_kc_denied_msg & 4) == 4) kc_record.victim iprintln(&"MISC_KDV_PROXY");
					}
				}
				else
				{
					if((level.ex_kc_denied_msg & 2) == 2) player thread dogtagConfirmHUD(&"MISC_KDV_HUD");
					if(isPlayer(kc_record.attacker) && (level.ex_kc_denied_msg & 4) == 4) player iprintln(&"MISC_KDV_SELF", [[level.ex_pname]](kc_record.attacker));
				}

				// for attacker
				if(isPlayer(kc_record.attacker))
				{
					if((level.ex_kc_denied_msg & 8) == 8) kc_record.attacker thread dogtagConfirmHUD(&"MISC_KDA_HUD");
					if((level.ex_kc_denied_msg & 16) == 16) kc_record.attacker iprintln(&"MISC_KDA_BY", [[level.ex_pname]](player));
					//if((level.ex_kc_denied_msg & 16) == 16) kc_record.attacker iprintln(&"MISC_KDA_SELF");
				}
			}

			lpplayerguid = player getGuid();
			lpplayernum = player getEntityNumber();
			lpplayerteam = player.pers["team"];
			lpplayername = player.name;
			logPrint("KD;" + lpplayerguid + ";" + lpplayernum + ";" + lpplayerteam + ";" + lpplayername + ";" + points_denied + "\n");
			
			// break out of loop
			break;
		}
		// pickup by attackers (the killer or his teammates)
		else
		{
			// if only attacker can collect, and you're not, deny pickup
			if(level.ex_kc == 1 && (!isPlayer(kc_record.attacker) || player != kc_record.attacker)) continue;

			// handle regular points
			points = kc_record.points + kc_record.reward;
			if(level.ex_kc == 2 && isPlayer(kc_record.attacker)) pointsto = kc_record.attacker;
			  else pointsto = player;

			pointsto.pers["score"] += points;
			pointsto.score += points;
			if(kc_record.reward) pointsto.pers["bonus"] += kc_record.reward;
			pointsto notify("update_playerscore_hud");
			if(!level.ex_teamplay) pointsto [[level.checkscorelimit]]();

			if(kc_record.isteamscore)
			{
				teamscore = getTeamScore(pointsto.pers["team"]);
				teamscore += points;
				setTeamScore(pointsto.pers["team"], teamscore);
				[[level.checkscorelimit]]();
			}

			// handle statistics
			if(isPlayer(pointsto))
			{
				pointsto.pers["kill"]++;
				for(i = 0; i < kc_record.attacker_stats.size; i++)
					pointsto.pers[kc_record.attacker_stats[i]]++;
			}
			if(isPlayer(kc_record.victim))
			{
				kc_record.victim.deaths++;
				kc_record.victim.pers["death"]++;
				for(i = 0; i < kc_record.victim_stats.size; i++)
					kc_record.victim.pers[kc_record.victim_stats[i]]++;
			}

			// sounds and messages
			if(level.ex_kc_confirm_msg)
			{
				// for collector
				if((level.ex_kc_confirm_msg & 1) == 1) player playLocalSound("kc_confirmed");
				if(player != kc_record.attacker)
				{
					if((level.ex_kc_confirm_msg & 2) == 2) player thread dogtagConfirmHUD(&"MISC_KCC_HUD");

					if(isPlayer(kc_record.attacker))
					{
						if((level.ex_kc_confirm_msg & 4) == 4) player iprintln(&"MISC_KCC_FOR", [[level.ex_pname]](kc_record.attacker));
						if((level.ex_kc_confirm_msg & 1) == 1) kc_record.attacker playLocalSound("kc_confirmed");
						if((level.ex_kc_confirm_msg & 2) == 2) kc_record.attacker thread dogtagConfirmHUD(&"MISC_KCA_PROXY_HUD");
						if((level.ex_kc_confirm_msg & 4) == 4) kc_record.attacker iprintln(&"MISC_KCA_BY", [[level.ex_pname]](player));
						//if((level.ex_kc_confirm_msg & 4) == 4) kc_record.attacker iprintln(&"MISC_KCA_PROXY");
					}
				}
				else
				{
					if((level.ex_kc_confirm_msg & 2) == 2) player thread dogtagConfirmHUD(&"MISC_KCA_HUD");
					if(isPlayer(kc_record.victim) && (level.ex_kc_confirm_msg & 4) == 4) player iprintln(&"MISC_KCA_SELF", [[level.ex_pname]](kc_record.victim));
				}

				// for victim
				if(isPlayer(kc_record.victim))
				{
					if((level.ex_kc_confirm_msg & 8) == 8) kc_record.victim thread dogtagConfirmHUD(&"MISC_KCV_HUD");
					if((level.ex_kc_confirm_msg & 16) == 16) kc_record.victim iprintln(&"MISC_KCV_BY", [[level.ex_pname]](player));
					//if((level.ex_kc_confirm_msg & 16) == 16) kc_record.victim iprintln(&"MISC_KCV_SELF");
				}
			}

			lpplayerguid = player getGuid();
			lpplayernum = player getEntityNumber();
			lpplayerteam = player.pers["team"];
			lpplayername = player.name;
			logPrint("KC;" + lpplayerguid + ";" + lpplayernum + ";" + lpplayerteam + ";" + lpplayername + ";" + points + "\n");

			// break out of loop
			break;
		}
	}

	// Notify timeout monitor to end
	self notify("kc_endtimeout");

	// Play pickup sound
	self playSound("kc_pickup");

	// Delete model
	kcFree(index);
}

dogtagConfirmHUD(notification)
{
	self notify("kc_update");
	waittillframeend;
	self endon("kc_update");

	if(!isDefined(self.ex_kc_hud)) self.ex_kc_hud = newClientHudElem(self);
	if(isDefined(self.ex_kc_hud))
	{
		self.ex_kc_hud.archived = true;
		self.ex_kc_hud.horzAlign = "fullscreen";
		self.ex_kc_hud.vertAlign = "fullscreen";
		self.ex_kc_hud.alignX = "center";
		self.ex_kc_hud.alignY = "middle";
		self.ex_kc_hud.x = 320;
		self.ex_kc_hud.y = 400;
		self.ex_kc_hud.fontscale = 1.0;
		self.ex_kc_hud.alpha = 1;
		self.ex_kc_hud setText(notification);

		wait( [[level.ex_fpstime]](.5) );

		if(isDefined(self.ex_kc_hud))
		{
			self.ex_kc_hud fadeOverTime(.5);
			self.ex_kc_hud.alpha = 0;
		}
	}
}

dogtagDebugHUD(index)
{
	if(!isDefined(self.ex_kc_debug)) self.ex_kc_debug = newClientHudElem(self);
	if(isDefined(self.ex_kc_debug))
	{
		self.ex_kc_debug.archived = true;
		self.ex_kc_debug.horzAlign = "fullscreen";
		self.ex_kc_debug.vertAlign = "fullscreen";
		self.ex_kc_debug.alignX = "center";
		self.ex_kc_debug.alignY = "middle";
		self.ex_kc_debug.x = 320;
		self.ex_kc_debug.y = 400;
		self.ex_kc_debug.fontscale = 2.0;
		self.ex_kc_debug.alpha = 1;
		self.ex_kc_debug setValue(index);

		wait( [[level.ex_fpstime]](1) );

		if(isDefined(self.ex_kc_debug)) self.ex_kc_debug destroy();
	}
}
